#--*-coding:utf-8-*--

class Alsetto < DiceBot

  def initialize
    super
    @sortType = 1 #ダイスのソート有
  end


  def prefixs
    ['\d+ALC\d+(x|\*)\d+', '\d+ALC(x|\*)\d+', '\d+ALC\d+', '\d+ALC',
     '\d+AL\d+(x|\*)\d+', '\d+AL(x|\*)\d+', '\d+AL\d+', '\d+AL']
  end

  def gameName
    '詩片のアルセット'
  end

  def gameType
    "Alsetto"
  end

  def getHelpMessage
    return <<MESSAGETEXT
・成功判定：[n]AL[X]　　　　・トライアンフ無し：[n]ALC[X]
・命中判定：[n]AL[X]*[p]　　・トライアンフ無し：[n]ALC[X]*[p]

ALコマンドはトライアンフの分だけ、自動で振り足し処理を行います。
「n」でダイス数を指定。必須。
「X」で目標値を指定。省略時は、デフォルトの「3」が使用されます。
「*[p]（x[p]）」を追記で攻撃力を指定。「*」と「x」のどちらでも可。
攻撃力指定で命中判定となり、成功数ではなく、ダメージを結果表示します。

ALCコマンドはトライアンフ無しで、成功数、ダメージを結果表示します。

【書式例】
・5AL → 5d6で目標値3。
・5ALC → 5d6で目標値3。トライアンフ無し。
・6AL2 → 6d6で目標値2。
・4AL*5 → 4d6で目標値3、攻撃力5の命中判定。
・7AL2x10 → 7d6で目標値2、攻撃力10の命中判定。
・8ALC4x5 → 8d6で目標値4、攻撃力5、トライアンフ無しの命中判定。
MESSAGETEXT
  end


  def rollDiceCommand(command)

    output =
      case command.upcase

      # ALCコマンド：命中判定
      when /(\d+)ALC(\d+)(x|\*)(\d+)$/i then
        rapid = $1.to_i
        target = $2.to_i
        if (target <= 0)
          target = 3
        end
        damage = $4.to_i
        checkRoll(rapid, target, damage, 1, 0)
      when /(\d+)ALC(x|\*)(\d+)$/i then
        rapid = $1.to_i
        target = 3
        damage = $3.to_i
        checkRoll(rapid, target, damage, 1, 0)

      # ALCコマンド：成功判定
      when /(\d+)ALC(\d+)$/i then
        rapid = $1.to_i
        target = $2.to_i
        damage = 0
        checkRoll(rapid, target, damage, 0, 0)
      when /(\d+)ALC$/i then
        rapid = $1.to_i
        target = 3
        damage = 0
        checkRoll(rapid, target, damage, 0, 0)

      # ALコマンド：命中判定
      when /(\d+)AL(\d+)(x|\*)(\d+)$/i then
        rapid = $1.to_i
        target = $2.to_i
        if (target <= 0)
          target = 3
        end
        damage = $4.to_i
        checkRoll(rapid, target, damage, 1, 1)
      when /(\d+)AL(x|\*)(\d+)$/i then
        rapid = $1.to_i
        target = 3
        damage = $3.to_i
        checkRoll(rapid, target, damage, 1, 1)

      # ALコマンド：成功判定
      when /(\d+)AL(\d+)$/i then
        rapid = $1.to_i
        target = $2.to_i
        damage = 0
        checkRoll(rapid, target, damage, 0, 1)
      when /(\d+)AL$/i then
        rapid = $1.to_i
        target = 3
        damage = 0
        checkRoll(rapid, target, damage, 0, 1)

      else
        nil
      end

    return output
  end

  def checkRoll(rapid, target, damage, damageFlag, criticalFlag)
    diceCount = 0
    criticalCount = 0
    rollCount = rapid
    result = ""

    while rollCount != 0
      dice, diceText = roll(rollCount, 6, @sortType)
      diceArray = diceText.split(/,/).collect{|i|i.to_i}

      diceCount2 = 0
      criticalCount2 = 0
      diceArray.each do |i|
        if(i <= target)
          diceCount += 1
          diceCount2 += 1
        end
        if(i == 1)
          criticalCount += 1
          criticalCount2 += 1
        end
      end

      if result == ""
        result = "#{diceCount2}[#{diceText}]"
      else
        result = "#{result}+#{diceCount2}[#{diceText}]"
      end

      if(criticalFlag == 0)
        rollCount = 0
      else
        rollCount = criticalCount2
      end
    end

    if(damageFlag == 0)
      if(criticalFlag == 0)
        result = "(#{rapid}D6\<\=#{target}) ＞ #{result} ＞ 成功数：#{diceCount}"
      else
        result = "(#{rapid}D6\<\=#{target}) ＞ #{result} ＞ 成功数：#{diceCount} / #{criticalCount}トライアンフ"
      end
    else
      resultDamage = diceCount * damage
      if(criticalFlag == 0)
        result = "(#{rapid}D6\<\=#{target}) ＞ #{result} ＞ Hits：#{diceCount}*#{damage} ＞ #{resultDamage}ダメージ"
      else
        result = "(#{rapid}D6\<\=#{target}) ＞ #{result} ＞ Hits：#{diceCount}*#{damage} ＞ #{resultDamage}ダメージ / #{criticalCount}トライアンフ"
      end
    end

    return result
  end

end
